/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.lang.schema.internal;

import com.google.gson.ExclusionStrategy;
import com.google.gson.FieldAttributes;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonSyntaxException;
import com.google.gson.TypeAdapter;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonWriter;

import com.redhat.thermostat.lang.schema.*;

import com.redhat.thermostat.lang.schema.annotations.Schema;
import com.redhat.thermostat.lang.schema.models.Timestamp;
import org.apache.felix.scr.annotations.Activate;
import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Service;

import java.io.IOException;

@Component(name = "JSONService")
@Service(JSONService.class)
public class JSONServiceImpl implements JSONService {

    private final GsonBuilder builder;

    /**
     * set mode to phase 1 or phase 2
     * phase 1- JSONService converts using existing JSON formats (with MongoDB $numberLong)
     * phase 2 - JSONService converts using "everything is a string" format (requires changes in web-client)
     */
    private static int PHASE_DEFAULT = 1;

    public JSONServiceImpl() {
        this(false);
    }

    public JSONServiceImpl(boolean prettyPrinting) {
        this(PHASE_DEFAULT, prettyPrinting);
    }

    JSONServiceImpl(int phase, boolean prettyPrinting) {

        builder = new GsonBuilder();
        if (prettyPrinting) {
            builder.setPrettyPrinting();
        }

        if (phase == 1) {
            registerPhase1TypeAdapters();
        } else {
            registerPhase2TypeAdapters();
        }

        builder.addSerializationExclusionStrategy(new AutoExclusionStategy());
    }

    private void registerPhase1TypeAdapters() {
        final LongTypeAdapter longTypeAdapter = new LongTypeAdapter();
        builder.registerTypeAdapter(long.class, longTypeAdapter);
        builder.registerTypeAdapter(Long.class, longTypeAdapter);

        final TimestampTypeAdapter timestampTypeAdapter = new TimestampTypeAdapter();
        builder.registerTypeAdapter(Timestamp.class, timestampTypeAdapter);
    }

    private void registerPhase2TypeAdapters() {

        final AutoAdapter autoAdapter = new AutoAdapter();

        builder.registerTypeAdapter(boolean.class, autoAdapter);
        builder.registerTypeAdapter(byte.class, autoAdapter);
        builder.registerTypeAdapter(short.class, autoAdapter);
        builder.registerTypeAdapter(int.class, autoAdapter);
        builder.registerTypeAdapter(long.class, autoAdapter);
        builder.registerTypeAdapter(double.class, autoAdapter);
        builder.registerTypeAdapter(float.class, autoAdapter);

        builder.registerTypeAdapter(Boolean.class, autoAdapter);
        builder.registerTypeAdapter(Byte.class, autoAdapter);
        builder.registerTypeAdapter(Short.class, autoAdapter);
        builder.registerTypeAdapter(Integer.class, autoAdapter);
        builder.registerTypeAdapter(Long.class, autoAdapter);
        builder.registerTypeAdapter(Double.class, autoAdapter);
        builder.registerTypeAdapter(Float.class, autoAdapter);
    }

    @Activate
    public void activate() {
    }

    @Override
    public String serialiase(Object target) {
        // TODO - consider registering this class with the schema registry
        return builder.create().toJson(target);
    }

    private class AutoAdapter extends TypeAdapter<Object> {

        @Override
        public void write(JsonWriter out, Object value) throws IOException {
            out.value("" + value);
        }

        @Override
        public Object read(JsonReader in) throws IOException {
            throw new UnsupportedOperationException("read() not yet implemented");
        }
    }

    private static class LongTypeAdapter extends TypeAdapter<Long> {

        private static final String NUMBER_LONG_IDENTIFIER = "$numberLong";

        @Override
        public void write(JsonWriter out, Long value) throws IOException {
            if (value == null) {
                out.nullValue();
            } else {
                out.beginObject();
                out.name(NUMBER_LONG_IDENTIFIER);
                out.value(value.toString());
                out.endObject();
            }
        }

        @Override
        public Long read(JsonReader in) throws IOException {
            in.beginObject();
            String name = in.nextName();
            if (!name.equals(NUMBER_LONG_IDENTIFIER)) {
                throw new JsonSyntaxException("Unexpected name: " + name);
            }
            long returnValue = in.nextLong();
            in.endObject();
            return returnValue;
        }
    }

    private static class TimestampTypeAdapter extends TypeAdapter<Timestamp> {

        private static final String NUMBER_LONG_IDENTIFIER = "$numberLong";

        @Override
        public void write(JsonWriter out, Timestamp value) throws IOException {
            if (value == null) {
                out.nullValue();
            } else {
                out.beginObject();
                out.name(NUMBER_LONG_IDENTIFIER);
                out.value("" + value.get());
                out.endObject();
            }
        }

        @Override
        public Timestamp read(JsonReader in) throws IOException {
            in.beginObject();
            String name = in.nextName();
            if (!name.equals(NUMBER_LONG_IDENTIFIER)) {
                throw new JsonSyntaxException("Unexpected name: " + name);
            }
            Timestamp returnValue = new Timestamp(in.nextLong());
            in.endObject();
            return returnValue;
        }
    }

    public class AutoExclusionStategy implements ExclusionStrategy {

        public boolean shouldSkipClass(Class<?> arg0) {
            return false;
        }

        public boolean shouldSkipField(FieldAttributes field) {
            return (field.getAnnotation(Schema.class) == null);
        }
    }
}
