/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.common.json;

import com.google.gson.JsonElement;
import com.google.gson.JsonParser;
import org.junit.Test;

import java.util.NoSuchElementException;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;

public class JsonUtilTest {

    private static final String JSON =
            "{\n" +
                    "  \"parents\" : [ \"homer\", \"marge\" ],\n" +
                    "  \"children\" : {\n" +
                    "    \"bart\" : {\n" +
                    "      \"title\" : \"son\",\n" +
                    "      \"age\" : \"14\"\n" +
                    "      },\n" +
                    "    \"lisa\" : {\n" +
                    "      \"title\" : \"daughter\",\n" +
                    "      \"age\" : \"12\"\n" +
                    "      },\n" +
                    "    \"maggie\" : {\n" +
                    "      \"title\" : \"daughter\",\n" +
                    "      \"age\" : \"1\"\n" +
                    "      }\n" +
                    "      \n" +
                    "  },\n" +
                    "  \"surname\" : \"simpsons\"\n" +
                    "}\n";

    @Test
    public void testCapitalize() {
        assertEquals("Fred", JsonUtil.capitalize("Fred"));
        assertEquals("Barney", JsonUtil.capitalize("barney"));
    }

    @Test
    public void testDeepCopy() {
        final JsonParser parser = new JsonParser();
        final JsonElement j0 = parser.parse(JSON);
        final JsonElement j1 = JsonUtil.deepCopy(j0);
        assertEquals(j0, j1);

        j1.getAsJsonObject().get("children").getAsJsonObject().get("bart").getAsJsonObject().addProperty("age", 99);
        assertNotEquals(j0, j1);
    }

    @Test
    public void testFetchObject() {
        final JsonParser parser = new JsonParser();
        final JsonElement j0 = parser.parse(JSON);
        final JsonElement bartAge = JsonUtil.fetch(j0, "children.bart.age");
        assertEquals(14, bartAge.getAsInt());
    }

    @Test(expected = NoSuchElementException.class)
    public void testFetchTooMuch() {
        final JsonParser parser = new JsonParser();
        final JsonElement j0 = parser.parse(JSON);
        JsonUtil.fetch(j0, "children.bart.age.overflow");
    }

    @Test(expected = NoSuchElementException.class)
    public void testFetchTooLittle() {
        final JsonParser parser = new JsonParser();
        final JsonElement j0 = parser.parse(JSON);
        JsonUtil.fetch(j0, "");
    }

    @Test
    public void testFetchArray() {
        final JsonParser parser = new JsonParser();
        final JsonElement j0 = parser.parse(JSON);
        final JsonElement homer = JsonUtil.fetch(j0, "parents.0");
        assertEquals("homer", homer.getAsString());
        final JsonElement marge = JsonUtil.fetch(j0, "parents.1");
        assertEquals("marge", marge.getAsString());
    }

    //@Test
    // disable until this syntax feature is implemented
    public void testFetchArraySquareBrackets() {
        final JsonParser parser = new JsonParser();
        final JsonElement j0 = parser.parse(JSON);
        final JsonElement homer = JsonUtil.fetch(j0, "parents[0]");
        assertEquals("homer", homer.getAsString());
        final JsonElement marge = JsonUtil.fetch(j0, "parents[1]");
        assertEquals("marge", marge.getAsString());
    }
}
